﻿/*
 * Instar Camera Interface
 * 
 * Hosted on codeplex: http://ici.codeplex.com
 * Camera devices: http://www.instar.de/
 * Author: Sebastian Kraemer, Sebastian.Kraemer@swKraemer.de
 *
 */

using System;


namespace InstarCameraInterface
{

    /// <summary>
    /// This class represents a single instar camera hardware device. 
    /// Depending from the used credentials the camera can be moved, the configuration changed or snapshots be done.
    /// </summary>
    public class InstarCamera
    {

        /// <summary>
        /// Creates a new instance of the InstarCamera object.
        /// </summary>
        /// <param name="camAddress">The protocol and address to connect to the camera device. For example http://192.168.0.20 or http://cameradevice.dyndns.org </param>
        /// <param name="Port">The port which shall be used for the connection</param>
        /// <param name="AdminUsername">the username of an administrative user account of the camera device</param>
        /// <param name="AdminPassword">the password for the administrative account</param>
        /// <param name="Timeout">Timeout in ms for connection to the camera device.</param>
        public InstarCamera(String camAddress, Int16 Port, String AdminUsername, String AdminPassword, int Timeout = 2500)
        {


            this.Connection = new InstarCameraConnectionDetail(this, camAddress, Port, Timeout, AdminUsername , AdminPassword );

            this.Configuration = new InstarCameraVarDeclarationList(this, "get_params.cgi");
            this.Configuration.Load();

            this.Users = new InstarCameraUserList(this);

            this.Status = new InstarCameraStatus(this);

            this.Snapshot = new InstarCameraSnapshot(this);

            this.Movement = new InstarCameraMovement(this);

            this.Network = new InstarCameraNetwork(this);

            this.CameraControl = new InstarCameraControl(this);

            this.Logfile = new InstarCameraLogfile(this);

            this.DateTime = new InstarCameraDateTime(this);
        
        }

        
        /// <summary>
        /// Connection details used for communicatioon wih instar camera device
        /// </summary>
        public InstarCameraConnectionDetail Connection {get; private set;}


        /// <summary>
        /// Image-Properties like contrast, brightness, resolutioon etc. of the instar camera device.
        /// </summary>
        public InstarCameraControl CameraControl { get; private set; }


        /// <summary>
        /// Allows the user to chance the cameras' angle of view.
        /// The camera has to support  this feature.
        /// </summary>
        public InstarCameraMovement Movement { get; private set; }

       
        /// <summary>
        /// Readonly access to the configuration of the camera
        /// </summary>
        public InstarCameraVarDeclarationList Configuration { get; private set; }

        
        /// <summary>
        /// Status settings of the camera device
        /// </summary>
        public InstarCameraStatus Status { get; private set; }
        
        
        /// <summary>
        /// User management of the camera device.
        /// </summary>
        public InstarCameraUserList Users { get; private set; }


        /// <summary>
        /// Restarts the camera device.
        /// </summary>
        /// <returns></returns>
        public Boolean Reboot()
        {

            return InstarCameraHelper.ExecuteHttpCommand("reboot.cgi", "", this);

        }

        
        /// <summary>
        /// Restores the camera factory default settings.
        /// </summary>
        /// <returns></returns>
        public Boolean RestoreFactoryDefaults()
        {
            return InstarCameraHelper.ExecuteHttpCommand("restore_factory.cgi", "", this);
        }


        /// <summary>
        /// Gets or sets the camera alias.
        /// </summary>
        public String Alias 
        {
            get {
                return this.Configuration.GetValueByName("alias").PropertyValue;
            }
            set {

                // Output a short debug message
                String msg = String.Format("set alias: alias={0}", value);
                System.Diagnostics.Debug.WriteLine(msg);

               

                if (value.Length <= 20)
                {
                    ///set_alias.cgi?alias=[&next_url=]
                    String parameter = String.Format("alias={0}", value);
                    InstarCameraHelper.ExecuteHttpCommand("set_alias.cgi", parameter, this);

                }
                else
                {
                    throw new InvalidOperationException("Value mustn't be longer than 20 chars.");

                }
            }
        }

    
        /// <summary>
        /// This class gives the possibiliy to get the current camera picture
        /// </summary>
        public InstarCameraSnapshot Snapshot { get; private set; }

        /// <summary>
        /// Read/Write access to to camera network configuration
        /// </summary>
        public InstarCameraNetwork Network { get; private set; }

        /// <summary>
        /// Read access to the cameras' logfile
        /// </summary>
        public InstarCameraLogfile Logfile { get; private set; }

        /// <summary>
        /// Read/Write access to the cameras' clock and some related details.
        /// </summary>
        public InstarCameraDateTime DateTime { get; private set; }

    }
}
