﻿/*
 * Instar Camera Interface
 * 
 * Hosted on codeplex: http://ici.codeplex.com
 * Camera devices: http://www.instar.de/
 * Author: Sebastian Kraemer, Sebastian.Kraemer@swKraemer.de
 *
 */
using System;

namespace InstarCameraInterface
{

    /// <summary>
    /// This class allows the user to read/write thhe values for camera resolution, brightness, contrast, flip/mirror and mode
    /// </summary>
    public class InstarCameraControl : InstarCameraClassBase
    {

        private enum CameraControlEnum {Resolution = 0, Brightness = 1, Contrast = 2, Mode = 3, FlipAndMirror = 5 }

        public enum InstarCameraResolution { x160y120 = 2, x320y240 = 8, x640y480 = 32 }

        public enum InstarCameraModeEnum { Hz50 = 0, Hz60 = 1, Outdoor = 2 }

        public enum InstarCameraFlipAndMorrorEnum { Default = 0, flip = 1, mirror = 2, flipAndMorror = 3 }


        /// <summary>
        /// Creates a new instance of the class.
        /// </summary>
        /// <param name="camera"></param>
        public InstarCameraControl(InstarCamera camera) :base(camera)
        { 
           
            this.CameraParams = new InstarCameraVarDeclarationList(this.Camera, "get_camera_params.cgi");
            this.CameraParams.Load();
            
        }

        /// <summary>
        /// Forces the class to reload the values from the camera on every read-access to a property in this class.
        /// true = force reload on every access
        /// false = use chached values
        /// </summary>
        public Boolean ForcePropertyReloadOnAccess {get; set;}

        /// <summary>
        /// Direct read-access to the values stored in the configuration file of the camera. 
        /// All values are represented as strings.
        /// </summary>
        public InstarCameraVarDeclarationList CameraParams {get; private set;}

        /// <summary>
        /// Refreshs the values of "CameraParams".
        /// </summary>
        public void Reload() { this.CameraParams.Reload(); }

        /// <summary>
        /// Returns if the class was able to retrieve data from the camera.
        /// </summary>
        public Boolean CanRead { get { return this.CameraParams.CanRead; } }
                
        /// <summary>
        /// gets or sets the resolution of the camera
        /// </summary>
        public InstarCameraResolution Resolution
        {
            get {
                
                // reload values if necessary
                if (ForcePropertyReloadOnAccess) this.CameraParams.Reload();

                // read resolution-property from property-list
                int value = int.Parse(this.CameraParams.GetValueByName("resolution").PropertyValue);

                // return casted value
                return (InstarCameraResolution)value;          
            }
            set
            {

                // create parameters fpr request
                String parameter = string.Format("param={0}&value={1}", (int)CameraControlEnum.Resolution, (int)value);
                
                // execute HTTP command
                InstarCameraHelper.ExecuteHttpCommand("camera_control.cgi", parameter, this.Camera);


            }
        }

        /// <summary>
        /// gets or sets the brightness of the camera
        /// </summary>
        public int Brightness
        {

            get {

                // reload values if necessary
                if (ForcePropertyReloadOnAccess) this.CameraParams.Reload();

                // read brightness-property from property-list
                int value = int.Parse(this.CameraParams.GetValueByName("brightness").PropertyValue);

                //return value
                return value;                     
            }
            set
            {

                // Check bounds
                if ((value >= 0) && (value <= 255))
                {
                    // bounds are correct, create parameter string
                    String parameter = string.Format("param={0}&value={1}", (int)CameraControlEnum.Brightness, value);

                    // execute http command
                    InstarCameraHelper.ExecuteHttpCommand("camera_control.cgi", parameter, this.Camera);
                }
                else {
                    // bounds are not correct
                    throw new ArgumentException("Value has to be between 0 and 255.");
                }
            }
        }

        /// <summary>
        /// gets or sets the contralst of the camera
        /// </summary>
        public int Contrast
        {
            get
            {
                // reload values if necessary
                if (ForcePropertyReloadOnAccess) this.CameraParams.Reload();

                // read contrast-property from property-list
                int value = int.Parse(this.CameraParams.GetValueByName("contrast").PropertyValue);

                // return value
                return value;          

            }
            set
            {

                // Check bounds
                if ((value >= 0) && (value <= 6))
                {
                    // bounds are correct, create parameter string
                    String parameter = string.Format("param={0}&value={1}", (int)CameraControlEnum.Contrast, value);

                    // execute HTTP command
                    InstarCameraHelper.ExecuteHttpCommand("camera_control.cgi", parameter, this.Camera);
                }
                else {
                    // bounds are not correct
                    throw new ArgumentException("Value hat to be between 0 and 6.");
                }
            }

        }

        /// <summary>
        /// gets or sets the mode of the camera
        /// </summary>
        public InstarCameraModeEnum Mode
        {
            get
            {
                // reload values if necessary
                if (ForcePropertyReloadOnAccess) this.CameraParams.Reload();

                // read mode-property from property-list
                int value = int.Parse(this.CameraParams.GetValueByName("mode").PropertyValue);

                // return casted value
                return (InstarCameraModeEnum)value;          
            }
            set
            {
                // create parameter string
                String parameter = string.Format("param={0}&value={1}", (int)CameraControlEnum.Mode, (int)value);

                // execute HTTP command
                InstarCameraHelper.ExecuteHttpCommand("camera_control.cgi", parameter, this.Camera);
            }
        }

        /// <summary>
        /// gets or sets the flip and mirror behaviour of the camera
        /// </summary>
        public InstarCameraFlipAndMorrorEnum FlipAndMirror
        {
            get
            {
                // reload values if necessary
                if (ForcePropertyReloadOnAccess) this.CameraParams.Reload();

                // read flip-property from property-list
                int value = int.Parse(this.CameraParams.GetValueByName("flip").PropertyValue);

                // return casted value
                return (InstarCameraFlipAndMorrorEnum)value;          
            }
            set
            {
                // create parameter string
                String parameter = string.Format("param={0}&value={1}", (int)CameraControlEnum.FlipAndMirror, (int)value);

                // execute HTTP command
                InstarCameraHelper.ExecuteHttpCommand("camera_control.cgi", parameter, this.Camera);
            }

        }
    }
}
