﻿/*
 * Instar Camera Interface
 * 
 * Hosted on codeplex: http://ici.codeplex.com
 * Camera devices: http://www.instar.de/
 * Author: Sebastian Kraemer, Sebastian.Kraemer@swKraemer.de
 *
 */
using System;

namespace InstarCameraInterface
{

    /// <summary>
    /// This class provides methods and properties to access the cameras' date/time configuration
    /// </summary>
    public class InstarCameraDateTime : InstarCameraClassBase
    {

        /// <summary>
        /// Creates a new instance of this class.
        /// </summary>
        /// <param name="camera"></param>
        public InstarCameraDateTime(InstarCamera camera) : base(camera) { }

        /*
         * /set_datetime.cgi?tz=&ntp_enable=&ntp_svr=[&now=&next_url=]
         * now: The count of seconds form 1970-1-1 0:0:0 to now;if use, the device will calibration time depend on it.
         * tz:set time zone. The current time zone settings contrast with the standard time zone GMT (Unit: Seconds)
         * ntp_enable:0:disable 1:enable
         * ntp_svr:ntp server length <= 64
         */

        /// <summary>
        /// returns the current time-zone-settings with the standard time zone GMT (Unit: Seconds)
        /// </summary>
        public int timezone { get { return int.Parse(this.Camera.Configuration.GetValueByName("tz").PropertyValue); } }

        /// <summary>
        /// returns if the network time protocol is enabled
        /// </summary>
        public Boolean NtpEnabled { get { return Boolean.Parse(this.Camera.Configuration.GetValueByName("ntp_enable").PropertyValue);  } }

        /// <summary>
        /// returns the network time protocol server
        /// </summary>
        public String NtpServer { get { return this.Camera.Configuration.GetValueByName("ntp_svr").PropertyValue; } }

        /// <summary>
        /// returns the current camera time
        /// </summary>
        /// <returns></returns>
        public DateTime Now()
        {

            /// reload the properties from the cam to get an up-to-date value
            this.Camera.Configuration.Reload();

            // fetch the now-value from the camera
            double secs = double.Parse(this.Camera.Configuration.GetValueByName("now").PropertyValue);

            // the now-value contains the seconds between the camera time an 01 Januars 1930
            DateTime cameraTime = new DateTime(1930, 1, 1, 0, 0, 0);
            cameraTime = cameraTime.AddSeconds(secs);

            // return the camera time 
            return cameraTime;

        }

        /// <summary>
        /// sets the systemtime to the camera
        /// </summary>
        public Boolean SetSystemTime()
        {
            return this.Set(this.timezone, this.NtpEnabled, this.NtpServer, true, System.DateTime.Now);
        }

        /// <summary>
        /// Sets time settings to the camera
        /// </summary>
        /// <param name="timezone">Timediff to GMT in seconds</param>
        /// <param name="ntpEnable">en/disable the network time protocol</param>
        /// <param name="ntpServer">the server for timesync</param>
        /// <returns></returns>
        private Boolean Set(int timezone, bool ntpEnable, String ntpServer)
        {
            return this.Set(timezone, ntpEnable, ntpServer, false, System.DateTime.Now);
        }

        /// <summary>
        /// Sets time and time settings to the camera
        /// </summary>
        /// <param name="timezone">Timediff to GMT in seconds</param>
        /// <param name="ntpEnable">en/disable the network time protocol</param>
        /// <param name="ntpServer">the server for timesync</param>
        /// <param name="time">the new camera time</param>
        /// <returns></returns>
        private Boolean Set(int timezone, bool ntpEnable, String ntpServer, DateTime time)
        {
            return this.Set(timezone, ntpEnable, ntpServer, true, time);
        }

        /// <summary>
        /// Sets time and time settings to the camera
        /// </summary>
        /// <param name="timezone">Timediff to GMT in seconds</param>
        /// <param name="ntpEnable">en/disable the network time protocol</param>
        /// <param name="ntpServer">the server for timesync</param>
        /// <param name="setNowParameter">Should the now-parameter be changed</param>
        /// <param name="time">the new camera time</param>
        /// <returns></returns>
        private Boolean Set(int timezone, bool ntpEnable, String ntpServer, Boolean setNowParameter, DateTime time)
        {
            // Url for call:
            // set_datetime.cgi?tz=&ntp_enable=&ntp_svr=[&now=&next_url=]


            if (ntpServer.Length > 64) throw new ArgumentException ("ntpServer mustn't be longer than 64 chars");

            // calculate the seconds between the set time and the 1st January 1930
            DateTime firstTime = new DateTime(1930, 1, 1, 0, 0, 0);
            double secDiff = (time - firstTime).TotalSeconds;

            // create the ntp_enable parameter value
            String strNtpEnable = (ntpEnable)? "1": "0";

            // create the command url
            string filename = "set_datetime.cgi";
            string parameter = String.Format("tz={0}&ntp_enable={1}&ntp_svr={2}", timezone, strNtpEnable, ntpServer);
            if (setNowParameter) parameter += String.Format("&now={0}", secDiff);

            // execute html command and return result
            return InstarCameraHelper.ExecuteHttpCommand(filename, parameter, this.Camera);
        
        }


    }
}
