﻿/*
 * Instar Camera Interface
 * 
 * Hosted on codeplex: http://ici.codeplex.com
 * Camera devices: http://www.instar.de/
 * Author: Sebastian Kraemer, Sebastian.Kraemer@swKraemer.de
 *
 */
using System;
using System.Collections.Generic;

namespace InstarCameraInterface
{

    /// <summary>
    /// This class represents a file containing key- and value records.
    /// One key-value pair per line. key and value are sperated by ':='
    /// Lines starting with '#' are comments.
    /// </summary>
    public class KeyValueFile
    {
        public KeyValueFile(String filename) {

            this.Filename = filename;

            // Output a short debug message
            String msg = String.Format("KeyValueFile: filename={0}", filename);
            System.Diagnostics.Debug.WriteLine(msg);
            
            // Check the validity of the arguments
            System.Diagnostics.Debug.Assert(!String.IsNullOrEmpty(filename), "Error: filename mustn't be empty.");
            System.Diagnostics.Debug.Assert(!this.Exists(), "Error: File not found.");
          
            // load key-value-pairs
            this.KeyValueEnumeration = this.GetKeyValueList();
        }

        /// <summary>
        /// Name/path of the key-value-file
        /// </summary>
        public String Filename { get; private set; }

        /// <summary>
        /// Checks if the file does exist.
        /// </summary>
        /// <returns></returns>
        public Boolean Exists() { return System.IO.File.Exists(this.Filename); }

        /// <summary>
        /// Key-value-list which is stored in the file.
        /// </summary>
        public IEnumerable<KeyValuePair<String, String>> KeyValueEnumeration { get; private set; }

        /// <summary>
        /// Loads the key-value-list.
        /// </summary>
        /// <returns></returns>
        private IEnumerable<KeyValuePair<String, String>> GetKeyValueList()
        {
        
            // create a list in which the key-value-records can be stored
            List<KeyValuePair<String, String>> ret = new List<KeyValuePair<String,String>>();

            // walk through all lines
            foreach (String Line in System.IO.File.ReadAllLines(this.Filename))
            {
                // continue if the line is a comment
                if (String.IsNullOrEmpty(Line) || Line.TrimStart().StartsWith("#")) continue;

                // continue if no seperator is found
                if (!Line.Contains(":=")) continue ;

                // split the line at the seperator
                String[] Parts = Line.Split(new String[] {":="},2, StringSplitOptions.None);

                // save key and value in the list
                ret.Add(new KeyValuePair<String, String>(Parts[0], Parts[1]));
            
            }
            

            return ret.ToArray();
        }

        /// <summary>
        /// Returns a value by its name.
        /// If name does not exist null will be returned
        /// </summary>
        /// <param name="Name"></param>
        /// <returns></returns>
        public String GetValueByName(String Name)
        {
            // Walk through all entries
            foreach (var KVP in this.KeyValueEnumeration)
            { 
                // if correct name is found return value
                 if (KVP.Key.Equals(Name)) return KVP.Value; 
            
            }
                 
            // correct name nor found, return null.
            return null;
        }
    }
}
